/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.sling;

import com.squeakysand.jcr.SimpleNode;
import com.squeakysand.jcr.StringValue;

import java.util.Arrays;
import java.util.List;

import javax.jcr.Node;

import org.apache.sling.api.SlingException;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.request.RequestPathInfo;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SlingUtils {

    private static final Logger LOG = LoggerFactory.getLogger(SlingUtils.class);
    private static final String SLING_RESOURCE_TYPE = "sling:resourceType";

    /**
     * Utility method for walking the content tree to find an ancestor node with a specific sling:resourceType value.
     */
    public static SimpleNode findAncestorBySlingResourceType(Node descendentNode, String slingResourceType) {
        SimpleNode simpleDescendentNode = SimpleNode.asSimpleNode(descendentNode);
        return simpleDescendentNode.getAncestorByPropertyValue(SLING_RESOURCE_TYPE, new StringValue(slingResourceType));
    }

    public static Resource getResource(String resourcePath, ResourceResolver resourceResolver) {
        Resource result = null;
        if ((resourcePath != null) && (resourceResolver != null)) {
            try {
                result = resourceResolver.getResource(resourcePath);
            } catch (SlingException e) {
                LOG.error(e.getMessage(), e);
            }
        }
        return result;
    }

    public static String getResourceType(Node node) {
        String result = null;
        if (node != null) {
            SimpleNode simpleNode = SimpleNode.asSimpleNode(node);
            result = simpleNode.getStringProperty(SLING_RESOURCE_TYPE);
        }
        LOG.debug("resource type for node {} is {}", node, result);
        return result;
    }

    public static List<String> getSelectors(SlingHttpServletRequest slingRequest) {
        RequestPathInfo requestPathInfo = slingRequest.getRequestPathInfo();
        String[] selectors = requestPathInfo.getSelectors();
        return Arrays.asList(selectors);
    }

    /**
     * Returns true if the specific selector string is present in the request and false otherwise.
     */
    public static Boolean isSelectorPresent(String selector, SlingHttpServletRequest slingRequest) {
        Boolean isPresent = Boolean.FALSE;
        String selectors = slingRequest.getRequestPathInfo().getSelectorString();
        if ((selectors != null) && selectors.contains(selector)) {
            isPresent = Boolean.TRUE;
        }
        return isPresent;
    }
}
