package com.squeakysand.sling.request;

import com.squeakysand.commons.lang.NullParameterException;
import com.squeakysand.commons.lang.StringUtils;
import com.squeakysand.commons.lang.TrimStrategy;

import org.apache.sling.api.request.RequestPathInfo;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.engine.impl.request.SlingRequestPathInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
public class RequestPathInfoFactory {

    private static class SimpleRequestPathInfo implements RequestPathInfo {

        private static final Logger LOG = LoggerFactory.getLogger(RequestPathInfoFactory.SimpleRequestPathInfo.class);

        private String path;
        private String selectorString;
        private String[] selectors;
        private String extension;
        private String suffix;

        /**
         * @param path
         */
        public SimpleRequestPathInfo(String urlPath) {
            if (urlPath == null) {
                throw new NullParameterException("urlPath");
            }
            if (!urlPath.startsWith("/")) {
                throw new IllegalArgumentException(String.format("urlPath must be an absolute path, was '%s'", urlPath));
            }
            // int lastDot = urlPath.lastIndexOf(".");
            if (urlPath.contains(".")) {
                // String extPlusSuffix = urlPath.substring(lastDot + 1);
                String extPlusSuffix = StringUtils.trimPrefix(urlPath, '.', TrimStrategy.AFTER_LAST);
                // urlPath = urlPath.substring(0, lastDot);
                urlPath = StringUtils.trimSuffix(urlPath, '.', TrimStrategy.AT_LAST);
                // int firstSlash = extPlusSuffix.indexOf("/");
                if (extPlusSuffix.contains("/")) {
                    // extension = extPlusSuffix.substring(0, firstSlash);
                    extension = StringUtils.trimSuffix(extPlusSuffix, '/', TrimStrategy.AT_FIRST);
                    // suffix = extPlusSuffix.substring(firstSlash);
                    suffix = StringUtils.trimPrefix(extPlusSuffix, '/', TrimStrategy.AT_FIRST);
                } else {
                    extension = extPlusSuffix;
                    suffix = null;
                }
                // int lastSlash = urlPath.lastIndexOf("/");
                String selectorStr;
                if (urlPath.contains("/")) {
                    // selectorStr = urlPath.substring(lastSlash + 1);
                    selectorStr = StringUtils.trimPrefix(urlPath, '/', TrimStrategy.AFTER_LAST);
                    // urlPath = urlPath.substring(0, lastSlash + 1);
                    urlPath = StringUtils.trimSuffix(urlPath, '/', TrimStrategy.AFTER_LAST);
                } else {
                    selectorStr = urlPath;
                }
                // int firstDot = selectorStr.indexOf(".");
                if (selectorStr.contains(".")) {
                    // urlPath = urlPath + selectorStr.substring(0, firstDot);
                    urlPath = urlPath + StringUtils.trimSuffix(selectorStr, '.', TrimStrategy.AT_FIRST);
                    // selectorStr = selectorStr.substring(firstDot + 1);
                    selectorStr = StringUtils.trimPrefix(selectorStr, '.', TrimStrategy.AFTER_FIRST);
                } else {
                    urlPath = urlPath + selectorStr;
                    selectorStr = null;
                }
                path = urlPath;
                if (selectorStr == null) {
                    selectorString = null;
                    selectors = new String[0];
                } else {
                    selectorString = selectorStr;
                    selectors = selectorStr.split("\\.");
                }
            } else {
                path = urlPath;
                selectorString = null;
                selectors = new String[0];
                extension = null;
                suffix = null;
            }
        }

        /**
         * @see org.apache.sling.api.request.RequestPathInfo#getExtension()
         */
        @Override
        public String getExtension() {
            return extension;
        }

        /**
         * @see org.apache.sling.api.request.RequestPathInfo#getResourcePath()
         */
        @Override
        public String getResourcePath() {
            return path;
        }

        /**
         * @see org.apache.sling.api.request.RequestPathInfo#getSelectors()
         */
        @Override
        public String[] getSelectors() {
            return selectors;
        }

        /**
         * @see org.apache.sling.api.request.RequestPathInfo#getSelectorString()
         */
        @Override
        public String getSelectorString() {
            return selectorString;
        }

        /**
         * @see org.apache.sling.api.request.RequestPathInfo#getSuffix()
         */
        @Override
        public String getSuffix() {
            return suffix;
        }

    }

    public static RequestPathInfo create(Resource resource) {
        return new SlingRequestPathInfo(resource);
    }

    public static RequestPathInfo create(String path) {
        return new SimpleRequestPathInfo(path);
    }

}
